#include "RTClib.h"
#include <LiquidCrystal_I2C.h>

RTC_DS1307 rtc;
LiquidCrystal_I2C lcd(0x27,16,2);

#define BUZZ 5
#define SET_ALARM 0
#define CLEAR_ALARM 1
#define GO_BACK 2
#define SETTING_ALARM 3
#define NR_BEEPS 4

#define BIN1 6
#define BIN2 9
#define AIN1 10
#define AIN2 11

typedef struct {
  volatile unsigned int hour;
  volatile unsigned int minute;
}alarm_t;

alarm_t alarm; 
int menu_option = 0;
int display_mode = SET_ALARM;
bool menu_mode = false;
bool alarm_set = false;
bool alarm_ringing = false;
bool setting_hour = false;
bool setting_minute = false;
int nr_interrupts = 0;

int buttonState = 0;
unsigned long lastInterrupt;
  
void setup () {
  Serial.begin(9600);
  lcd.init();
  lcd.backlight();
  pinMode(BUZZ, OUTPUT);
  pinMode(4, INPUT_PULLUP);
  pinMode(AIN1,OUTPUT);
  pinMode(AIN2,OUTPUT);
  pinMode(BIN1,OUTPUT);
  pinMode(BIN2,OUTPUT);
  attachInterrupt(digitalPinToInterrupt(2), menu_function, RISING);
  attachInterrupt(digitalPinToInterrupt(3), switch_function, RISING);

#ifndef ESP8266
  while (!Serial); // wait for serial port to connect. Needed for native USB
#endif

  if (! rtc.begin()) {
    Serial.println("Couldn't find RTC");
    Serial.flush();
    while (1) delay(10);
  }
}

bool is_alarm_time() {
  DateTime now = rtc.now();
  if (now.hour() == alarm.hour && now.minute() == alarm.minute) {
    return true;
  }
  return false;
}

// The menu_function, attached to the first interrupt / button, is
// equivalent to a  "SELECT" type of button which either chooses
// the selected option displayed on the lcd or advances the alarm
// setting process further 

void menu_function() {
  if (millis() - lastInterrupt > 500) {
    if (alarm_ringing == true) {
      alarm_ringing = false;
      alarm_set = false;
      menu_mode = false;
      display_mode = SET_ALARM;
    }
    else if (display_mode == SETTING_ALARM) {
      if (setting_hour == true) {
        setting_hour = false;
        setting_minute = true;
      }
      else if (setting_minute = true) {
        setting_minute = false;
        alarm_set = true;
        display_mode = SET_ALARM;
      }
    }
    else if (menu_mode == false) {
      menu_mode = true;
      display_mode = SET_ALARM;
    }
    else {
      if (display_mode == GO_BACK) {
        menu_mode = false;
      }
      else if (display_mode == CLEAR_ALARM) {
        alarm_set = false;
        menu_mode = false;
      }
      else if (display_mode == SET_ALARM) {
        display_mode = SETTING_ALARM;
        menu_mode = false;
        alarm.hour = 0;
        alarm.minute = 0;
        setting_hour = true;
      }
    }
    lastInterrupt = millis();
  }
}

// The switch_function, attached to the second interrupt / button, is
// equivalent to a  "NEXT" type of button which either switches
// the selected option displayed on the lcd or increases the alarm
// time hour / minute counters 

void switch_function() {
  if (millis() - lastInterrupt > 500) {
    if (display_mode == SETTING_ALARM) {
      if (setting_hour == true) {
        alarm.hour++;
        if (alarm.hour == 24) {
          alarm.hour = 0;
        }
      }
      else if (setting_minute == true) {
        alarm.minute++;
        if (alarm.minute == 60) {
          alarm.minute = 0;
        }
      }
    }
    else if (menu_mode == true) {
      if (display_mode == SET_ALARM) {
        display_mode = CLEAR_ALARM;
      }
      else if (display_mode == CLEAR_ALARM) {
        display_mode = GO_BACK;
      }
      else if (display_mode == GO_BACK) {
        display_mode = SET_ALARM;
      } 
    }
    lastInterrupt = millis();
  }
}

// The display_setting_alarm is a function which runs in the loop
// while the user is setting up his alarm

void display_setting_alarm() {
  lcd.setCursor(0,1);
  if (alarm.hour < 10) {
    lcd.print('0');
    lcd.print(alarm.hour, DEC);
    lcd.print(':');
  }
  else {
    lcd.print(alarm.hour, DEC);
    lcd.print(':');
  }
  if (alarm.minute < 10) {
    lcd.print('0');
    lcd.print(alarm.minute, DEC);
  }
  else lcd.print(alarm.minute, DEC);
  delay(200);
  if (setting_hour == true) {
    lcd.setCursor(0,1);
    lcd.print("  ");
    lcd.print(':');
    if (alarm.minute < 10) {
      lcd.print('0');
      lcd.print(alarm.minute, DEC);
    }
    else lcd.print(alarm.minute, DEC);
  }
  else if (setting_minute == true) {
    lcd.setCursor(0,1);
    if (alarm.hour < 10) {
      lcd.print('0');
      lcd.print(alarm.hour, DEC);
      lcd.print(':');
    }
    else {
      lcd.print(alarm.hour, DEC);
      lcd.print(':');
    }
    lcd.print("  ");
  }
  delay(100);
}

//Functia display_current_time afiseaza data si ora curenta
//stocate in RTC

void display_current_time() {
    DateTime now = rtc.now();
    lcd.setCursor(0,0);
    lcd.print(now.year());
    lcd.print('/');
    if (now.month() < 10) {
      lcd.print('0');
      lcd.print(now.month(), DEC);
      lcd.print('/');
    }
    else {
      lcd.print(now.month(), DEC);
      lcd.print('/');
    }
    if (now.day() < 10) {
      lcd.print('0');
      lcd.print(now.day(), DEC);
      lcd.print("   ");
    }
    else {
      lcd.print(now.day(), DEC);
      lcd.print("   ");
    }

    lcd.setCursor(0,1);
    if (now.hour() < 10) {
      lcd.print('0');
      lcd.print(now.hour(), DEC);
      lcd.print(':');
    }
    else {
      lcd.print(now.hour(), DEC);
      lcd.print(':');
    }
    if (now.minute() < 10) {
      lcd.print('0');
      lcd.print(now.minute(), DEC);
    }
    else lcd.print(now.minute(), DEC);
}

void display_set_alarm() {
    lcd.setCursor(0,0);
    lcd.print("SET ALARM  ");
}

void display_clear_alarm() {
    lcd.setCursor(0,0);
    lcd.print("CLEAR ALARM");
}

void display_go_back() {
    lcd.setCursor(0,0);
    lcd.print("GO BACK    ");
}

//Functia ring_alarm declanseaza buzzerul cat timp variabila boleana
//corespunzatoare conditiei de pornire a alarmei

void ring_alarm(){
  for (int i = 0; i < NR_BEEPS; i++) {
  analogWrite(BUZZ, 255);
  delay(100);
  analogWrite(BUZZ, 0);
  delay(10);
  }
  delay(100);
}

// Functia run_motors activeaza motoarele odata ce alarma incepe sa sune

void run_motors() {
    digitalWrite(AIN1,HIGH); 
    digitalWrite(AIN2,LOW);
    digitalWrite(BIN1,HIGH); 
    digitalWrite(BIN2,LOW);
}

// Functia stop_motors opreste motoarele odata ce utilizatorul opreste alarma

void stop_motors() {
    digitalWrite(AIN1,LOW); 
    digitalWrite(AIN2,LOW);
    digitalWrite(BIN1,LOW); 
    digitalWrite(BIN2,LOW);
}

void loop () {

    buttonState = digitalRead(3);
    Serial.println(alarm.hour);
    //Serial.println(nr_interrupts);

    if (menu_mode == false) {
      if (display_mode == SETTING_ALARM) {
        display_setting_alarm();
      }
      else display_current_time();
    }
    else {
      if (display_mode == SET_ALARM) {
        display_set_alarm();
      }
      else if (display_mode == CLEAR_ALARM) {
        display_clear_alarm();
      }
      else if (display_mode == GO_BACK) {
        display_go_back();
      }
    }

    if (alarm_set == true && is_alarm_time() == true) {
      alarm_ringing = true;
    }

    if (alarm_ringing == true) {
      ring_alarm();
      run_motors();
    }
    else {
      stop_motors();
    }

    
}
